#include <stdio.h>
#include <windows.h>
#include <setupapi.h>
#include <devguid.h>
#include <regstr.h>
#include <tchar.h>

#include "resource.h"

#include "FCDHidCmd.h"
#include "Registry.h"

static HINSTANCE _hInst=NULL;
static HWND _hDlg=NULL;
static TCHAR _szVIDPID[]=_T("Vid_04d8&Pid_fb31");

typedef struct 
{
	TCHAR *pszDesc;
	unsigned __int8 u8Val;

} COMBOITEMSTRUCT;

typedef struct
{
	unsigned __int8 u8CommandSet;
	unsigned __int8 u8CommandGet;
	int nIdxDefault;
	int nID;
	const COMBOITEMSTRUCT *pacis;
} COMBOSTRUCT;

static const COMBOITEMSTRUCT _cisRFFilter[]=
{
	{_T("150kHz-4MHz"),TRFE_0_4},
	{_T("4MHz-8MHz"),TRFE_4_8},
	{_T("8MHz-16MHz"),TRFE_8_16},
	{_T("16MHz-32MHz"),TRFE_16_32},
	{_T("32MHz-75MHz"),TRFE_32_75},
	{_T("75MHz-125MHz"),TRFE_75_125},
	{_T("125MHz-250MHz"),TRFE_125_250},
	{_T("144MHz-146MHz"),TRFE_145},
	{_T("420MHz-875MHz"),TRFE_410_875},
	{_T("430MHz-440MHz"),TRFE_435},
	{_T("875MHz-1900MHz"),TRFE_875_2000},
	{NULL,0}
};

static const COMBOITEMSTRUCT _cisIFFilter[]=
{
	{_T("200kHz"),TIFE_200KHZ},
	{_T("300kHz"),TIFE_300KHZ},
	{_T("600kHz"),TIFE_600KHZ},
	{_T("1536kHz"),TIFE_1536KHZ},
	{_T("5MHz"),TIFE_5MHZ},
	{_T("6MHz"),TIFE_6MHZ},
	{_T("7MHz"),TIFE_7MHZ},
	{_T("8MHz"),TIFE_8MHZ},
	{NULL,0}
};

static COMBOSTRUCT _acs[]=
{
	FCD_HID_CMD_SET_RF_FILTER,   FCD_HID_CMD_GET_RF_FILTER,    0,IDC_COMBORFFILTER,   _cisRFFilter,
	FCD_HID_CMD_SET_IF_FILTER,   FCD_HID_CMD_GET_IF_FILTER,    0,IDC_COMBOIFFILTER,   _cisIFFilter,
	0,0,0,-1,NULL
};

static void PopulateCombo(HWND hDlg,int nID,int nIdxDefault,const COMBOITEMSTRUCT *pcis)
{
	SendDlgItemMessage(hDlg,nID,CB_RESETCONTENT,0,0);

	while (pcis->pszDesc!=NULL)
	{
		SendDlgItemMessage(hDlg,nID,CB_ADDSTRING,0,(LPARAM)pcis->pszDesc);
		pcis++;
	}
	SendDlgItemMessage(hDlg,nID,CB_SETCURSEL,(WPARAM)nIdxDefault,0);
}

static void PopulateCombos(HWND hDlg)
{
	COMBOSTRUCT *pcs=_acs;

	while (pcs->pacis!=NULL)
	{
		const COMBOITEMSTRUCT *pcis=pcs->pacis;

		PopulateCombo(hDlg,pcs->nID,pcs->nIdxDefault,pcis);

		pcs++;
	}
}

static BOOL HIDOpen(PTCHAR pszDevSearchID,PTCHAR pszDevID,PHANDLE phWrite,PHANDLE phRead,int nDevIDSize)
{
	GUID InterfaceClassGuid = {0x4d1e55b2, 0xf16f, 0x11cf, 0x88, 0xcb, 0x00, 0x11, 0x11, 0x00, 0x00, 0x30}; 
	HDEVINFO hDevInfo;
	SP_DEVINFO_DATA DeviceInfoData;
	DWORD i;
	PTCHAR pszDevSearchIDLower=_tcsdup(pszDevSearchID);

	_tcslwr_s(pszDevSearchIDLower,_tcslen(pszDevSearchIDLower)+1);

	// Create a HDEVINFO with all present devices.
	hDevInfo = SetupDiGetClassDevs
	(
		&InterfaceClassGuid,
		0, // Enumerator
		0,
		DIGCF_PRESENT | DIGCF_DEVICEINTERFACE 
	);
   
	if (hDevInfo == INVALID_HANDLE_VALUE)
	{
		// Insert error handling here.
		return FALSE;
	}
   
	// Enumerate through all devices in Set.
    DeviceInfoData.cbSize = sizeof(SP_DEVINFO_DATA);
	for (i=0;SetupDiEnumDeviceInfo(hDevInfo,i,&DeviceInfoData);i++)
	{
		DWORD DataT;
		LPTSTR buffer = NULL;
		DWORD buffersize = 0;

		while
		(
			!SetupDiGetDeviceRegistryProperty
			(
				hDevInfo,
				&DeviceInfoData,
				SPDRP_HARDWAREID,
				&DataT,
				(PBYTE)buffer,
				buffersize,
				&buffersize
			)
		)
		{
			if (GetLastError() == ERROR_INSUFFICIENT_BUFFER)
			{
				// Change the buffer size.
				if (buffer) 
				{
					LocalFree(buffer);
				}
				// Double the size to avoid problems on 
				// W2k MBCS systems per KB 888609. 
				buffer=LocalAlloc(LPTR,buffersize * 2);
			}
			else
			{
				// Insert error handling here.
				break;
			}
		}

		_tcslwr_s(buffer,nDevIDSize);
       
		if (_tcsstr(buffer,pszDevSearchIDLower)!=NULL)
		{
			// We have found the device, now lets try to open a handles to it

			SP_DEVICE_INTERFACE_DATA dids;

			dids.cbSize = sizeof(SP_DEVICE_INTERFACE_DATA);
			if(!SetupDiEnumDeviceInterfaces(hDevInfo, NULL, &InterfaceClassGuid, i, &dids))
			{
				break;
			}

			{
				SP_DEVICE_INTERFACE_DETAIL_DATA didds;
				PSP_DEVICE_INTERFACE_DETAIL_DATA pdidds=&didds;
				DWORD cb;

				pdidds->cbSize=sizeof(SP_DEVICE_INTERFACE_DETAIL_DATA);
				SetupDiGetDeviceInterfaceDetail(hDevInfo,&dids,NULL,0,&cb,NULL);	
				if ((pdidds=malloc(cb))==NULL)
				{
					break; // uh-oh
				}
				pdidds->cbSize=sizeof(SP_DEVICE_INTERFACE_DETAIL_DATA);
				SetupDiGetDeviceInterfaceDetail(hDevInfo,&dids,pdidds,cb,NULL,NULL);

				*phWrite=CreateFile(pdidds->DevicePath,GENERIC_WRITE,FILE_SHARE_READ | FILE_SHARE_WRITE,NULL,OPEN_EXISTING,0,0);
				if (GetLastError()!=ERROR_SUCCESS)
				{
					break;
				}
				*phRead=CreateFile(pdidds->DevicePath,GENERIC_READ,FILE_SHARE_READ | FILE_SHARE_WRITE,NULL,OPEN_EXISTING,0,0);
				if (GetLastError()!=ERROR_SUCCESS)
				{
					break;
				}
			}
			SetupDiDestroyDeviceInfoList(hDevInfo);
			free(pszDevSearchIDLower);
			_tcscpy_s(pszDevID,nDevIDSize,buffer);
			return TRUE;
		}
       
		if (buffer)
		{
			LocalFree(buffer);
		}
	}
	if ( GetLastError()!=NO_ERROR && GetLastError()!=ERROR_NO_MORE_ITEMS )
	{
	   return FALSE;
	}
	SetupDiDestroyDeviceInfoList(hDevInfo);
	free(pszDevSearchIDLower);
	return FALSE;
}

static void HIDClose(HANDLE hWrite,HANDLE hRead)
{
	CloseHandle(hWrite);
	CloseHandle(hRead);
}

static unsigned char HIDSetFreqOld(HANDLE hWrite,HANDLE hRead,int nFreq)
{
	DWORD dwBytesWritten=0;
	DWORD dwBytesRead=0;
	unsigned char au8BufOut[65]; // endpoint size + 1
	unsigned char au8BufIn[65]; // endpoint size + 1

	au8BufOut[0]=0; // First byte is report ID. Ignored by HID Class firmware as only config'd for one report
	au8BufOut[1]=100;
	au8BufOut[2]=(unsigned char)nFreq;
	au8BufOut[3]=(unsigned char)(nFreq>>8);
	au8BufOut[4]=(unsigned char)(nFreq>>16);
	WriteFile(hWrite,au8BufOut,65,&dwBytesWritten,0);
	ReadFile(hRead,au8BufIn,65,&dwBytesRead,0);
	return au8BufIn[2];
}

static unsigned char HIDSetFreq(HANDLE hWrite,HANDLE hRead,unsigned int uFreq)
{
	DWORD dwBytesWritten=0;
	DWORD dwBytesRead=0;
	unsigned char au8BufOut[65]; // endpoint size + 1
	unsigned char au8BufIn[65]; // endpoint size + 1

	uFreq*=1000U;

	au8BufOut[0]=0; // First byte is report ID. Ignored by HID Class firmware as only config'd for one report
	au8BufOut[1]=101;
	au8BufOut[2]=(unsigned char)uFreq;
	au8BufOut[3]=(unsigned char)(uFreq>>8);
	au8BufOut[4]=(unsigned char)(uFreq>>16);
	au8BufOut[5]=(unsigned char)(uFreq>>24);
	WriteFile(hWrite,au8BufOut,65,&dwBytesWritten,0);
	ReadFile(hRead,au8BufIn,65,&dwBytesRead,0);
	return au8BufIn[2];
}

static unsigned char HIDResetFCD(HANDLE hWrite,HANDLE hRead)
{
	DWORD dwBytesWritten=0;
	DWORD dwBytesRead=0;
	unsigned char au8BufOut[65]; // endpoint size + 1
	unsigned char au8BufIn[65]; // endpoint size + 1

	au8BufOut[0]=0; // First byte is report ID. Ignored by HID Class firmware as only config'd for one report
	au8BufOut[1]=FCD_RESET;
	WriteFile(hWrite,au8BufOut,65,&dwBytesWritten,0);
	ReadFile(hRead,au8BufIn,65,&dwBytesRead,0);
	return au8BufIn[2];
}

static void Message(PTCHAR psz)
{
	SendMessage(GetDlgItem(_hDlg,IDC_EDITMESSAGE),EM_REPLACESEL,(WPARAM)FALSE,(LPARAM)psz);
}

static void MessageClear(void)
{
	if (_hDlg!=NULL)
	{
		SetDlgItemText(_hDlg,IDC_EDITMESSAGE,_T(""));
	}
}

static BOOL HIDReadCommand(HANDLE hWrite,HANDLE hRead,unsigned __int8 u8Cmd,unsigned __int8 *pu8Data,unsigned __int8 u8Len)
{
	DWORD dwBytesWritten=0;
	DWORD dwBytesRead=0;
	unsigned char au8BufOut[65]; // endpoint size + 1
	unsigned char au8BufIn[65]; // endpoint size + 1

	// Clean out any outstanding reads
	
	au8BufOut[0]=0; // First byte is report ID. Ignored by HID Class firmware as only config'd for one report
	au8BufOut[1]=u8Cmd;
	Sleep(10);
	WriteFile(hWrite,au8BufOut,65,&dwBytesWritten,0);
	ReadFile(hRead,au8BufIn,65,&dwBytesRead,0);
	memcpy(pu8Data,au8BufIn+3,u8Len);
	return au8BufIn[2];
}

static BOOL HIDQuery(HANDLE hWrite,HANDLE hRead,unsigned __int8 *pu8Data,unsigned __int8 u8Len)
{
	DWORD dwBytesWritten=0;
	DWORD dwBytesRead=0;
	unsigned char au8BufOut[65]; // endpoint size + 1
	unsigned char au8BufIn[65]; // endpoint size + 1

	// Clean out any outstanding reads
	
	au8BufOut[0]=0; // First byte is report ID. Ignored by HID Class firmware as only config'd for one report
	au8BufOut[1]=FCD_HID_CMD_QUERY;
	Sleep(10);
	WriteFile(hWrite,au8BufOut,65,&dwBytesWritten,0);
	ReadFile(hRead,au8BufIn,65,&dwBytesRead,0);
	memcpy(pu8Data,au8BufIn+3,u8Len);
	return au8BufIn[2];
}

static BOOL CheckVersion(HANDLE hWrite,HANDLE hRead,char *pszVersion)
{
	char szVersionRead[60];

	if (HIDQuery(hWrite,hRead,szVersionRead,60)!=TRUE)
	{
		if (strncmp(szVersionRead,"FCDAPP",6)==0)
		{

		}
		else
		{
			Message(_T("\r\nError, FCD not in application mode."));
		}
	}
	else
	{
		Message(_T("\r\nError, FCD firmware too low, Query command not supported."));
	}
}

static BOOL ReadCommand(unsigned __int8 u8Cmd,unsigned __int8 *pu8Data,unsigned __int8 u8Len)
{
	TCHAR sz[200];
	TCHAR szMsg[400];
	BOOL b;
	HANDLE hWrite=INVALID_HANDLE_VALUE;
	HANDLE hRead=INVALID_HANDLE_VALUE;

	MessageClear();
	b=HIDOpen(_szVIDPID,sz,&hWrite,&hRead,sizeof(sz)/sizeof(TCHAR));
	_stprintf_s(szMsg,sizeof(szMsg)/sizeof(TCHAR),_T("HIDOpen of %s returns %d"),_szVIDPID,b);
	Message(szMsg);
	if (b)
	{
		unsigned __int8 u8;

		Message(_T("\r\nSuccess, FCD found."));
		u8=HIDReadCommand(hWrite,hRead,u8Cmd,pu8Data,u8Len);
		_stprintf_s(szMsg,sizeof(szMsg)/sizeof(TCHAR),_T("\r\nHIDReadCommand returns %d"),u8);
		Message(szMsg);
		if (u8==1)
		{
			_stprintf_s(szMsg,sizeof(szMsg)/sizeof(TCHAR),_T("\r\nSuccess, FCD read command, value: %d"),*pu8Data);
			Message(szMsg);
		}
		else
		{
			Message(_T("\r\nError, FCD read command."));
		}
	}
	else
	{
		Message(_T("\r\nError, FCD not found: is FCD correctly installed?"));
	}
	HIDClose(hWrite,hRead);

	return b;
}

static BOOL HIDWriteCommand(HANDLE hWrite,HANDLE hRead,unsigned __int8 u8Cmd,unsigned __int8 *pu8Data,unsigned __int8 u8Len)
{
	DWORD dwBytesWritten=0;
	DWORD dwBytesRead=0;
	unsigned char au8BufOut[65]; // endpoint size + 1
	unsigned char au8BufIn[65]; // endpoint size + 1

	// Clean out any outstanding reads
	au8BufOut[0]=0; // First byte is report ID. Ignored by HID Class firmware as only config'd for one report
	au8BufOut[1]=u8Cmd;
	memcpy(au8BufOut+2,pu8Data,u8Len);
	Sleep(10);
	WriteFile(hWrite,au8BufOut,65,&dwBytesWritten,0);
	ReadFile(hRead,au8BufIn,65,&dwBytesRead,0);
	return au8BufIn[2];
}

static void WriteCommand(unsigned __int8 u8Cmd,unsigned __int8 *pu8Data,unsigned __int8 u8Len)
{
	TCHAR sz[200];
	TCHAR szMsg[400];
	BOOL b;
	HANDLE hWrite=INVALID_HANDLE_VALUE;
	HANDLE hRead=INVALID_HANDLE_VALUE;

	MessageClear();
	b=HIDOpen(_szVIDPID,sz,&hWrite,&hRead,sizeof(sz)/sizeof(TCHAR));
	_stprintf_s(szMsg,sizeof(szMsg)/sizeof(TCHAR),_T("HIDOpen of %s returns %d"),_szVIDPID,b);
	Message(szMsg);
	if (b)
	{
		unsigned __int8 u8;

		Message(_T("\r\nSuccess, FCD found."));
		u8=HIDWriteCommand(hWrite,hRead,u8Cmd,pu8Data,u8Len);
		_stprintf_s(szMsg,sizeof(szMsg)/sizeof(TCHAR),_T("\r\nHIDWriteCommand returns %d"),u8);
		Message(szMsg);
		if (u8==1)
		{
			Message(_T("\r\nSuccess, FCD write register."));
		}
		else
		{
			Message(_T("\r\nError, FCD read register."));
		}
	}
	else
	{
		Message(_T("\r\nError, FCD not found: is FCD correctly installed?"));
	}
	HIDClose(hWrite,hRead);
}

static BOOL ReadDevice(HWND hDlg)
{
	COMBOSTRUCT *pcs=_acs;
//	unsigned __int16 au16[50];
	int i=0;
	BOOL b=FALSE;

	while (pcs->pacis!=NULL)
	{
		unsigned __int8 u8;
		const COMBOITEMSTRUCT *pcis=pcs->pacis;
		int nIdx=0;

		ReadCommand(pcs->u8CommandGet,&u8,1);

		// Try to find the index to the register field value
		while (pcis!=NULL && pcis->pszDesc!=NULL && pcis->u8Val!=u8)
		{
			nIdx++;
			pcis++;
		}

		if (pcis!=NULL && pcis->pszDesc!=NULL)
		{
			SendDlgItemMessage(hDlg,pcs->nID,CB_SETCURSEL,(WPARAM)nIdx,0);
		}
		else
		{
			SendDlgItemMessage(hDlg,pcs->nID,CB_SETCURSEL,(WPARAM)-1,0); // clear the combo box if cannot find
		}
		pcs++;
		i++;
	}
	{
		unsigned __int8 u8;
		unsigned __int32 u32;
		TCHAR sz[30];

		ReadCommand(FCD_HID_CMD_GET_BIAS_TEE,&u8,1);
		SendDlgItemMessage(hDlg,IDC_CHECKBIASTEE,BM_SETCHECK,u8 ? BST_CHECKED : BST_UNCHECKED,0);

		ReadCommand(FCD_HID_CMD_GET_LNA_GAIN,&u8,1);
		SendDlgItemMessage(hDlg,IDC_CHECKLNAENABLE,BM_SETCHECK,u8 ? BST_CHECKED : BST_UNCHECKED,0);

		ReadCommand(FCD_HID_CMD_GET_MIXER_GAIN,&u8,1);
		SendDlgItemMessage(hDlg,IDC_CHECKMIXERGAIN,BM_SETCHECK,u8 ? BST_CHECKED : BST_UNCHECKED,0);

		ReadCommand(FCD_HID_CMD_GET_IF_GAIN,&u8,1);
		SendDlgItemMessage(hDlg,IDC_SCROLLBARFINEIFGAIN,SBM_SETPOS,u8,0);

		{
			TCHAR sz[30]=_T("0");

			SetDlgItemText(hDlg,IDC_EDITIFGAIN,sz);
		}

		if (!ReadCommand(FCD_HID_CMD_GET_FREQUENCY_HZ,(unsigned __int8 *)&u32,sizeof(u32)))
		{
			u32=0;
		}
		else
		{
			b=TRUE;
		}
		_stprintf_s(sz,sizeof(sz)/sizeof(TCHAR),_T("%d"),u32);
		SetDlgItemText(hDlg,IDC_EDITGETFREQUENCY,sz);
	}
	return b;
}

static void SetFreq(int nFreq,BOOL bMsgClear)
{
	TCHAR sz[200];
	TCHAR szMsg[400];
	BOOL b;
	HANDLE hWrite=INVALID_HANDLE_VALUE;
	HANDLE hRead=INVALID_HANDLE_VALUE;

	if (bMsgClear)
	{
		MessageClear();
	}
	b=HIDOpen(_szVIDPID,sz,&hWrite,&hRead,sizeof(sz)/sizeof(TCHAR));
	_stprintf_s(szMsg,sizeof(szMsg)/sizeof(TCHAR),_T("HIDOpen of %s returns %d"),_szVIDPID,b);
	Message(szMsg);
	if (b)
	{
		unsigned int u8;

		Message(_T("\r\nSuccess, FCD found."));
		u8=HIDSetFreq(hWrite,hRead,nFreq);
		_stprintf_s(szMsg,sizeof(szMsg)/sizeof(TCHAR),_T("\r\nHIDSetFreq of %d returns %d"),nFreq,u8);
		Message(szMsg);
		if (u8==1)
		{
			ReadDevice(_hDlg);
			Message(_T("\r\nSuccess, FCD frequency set."));
		}
		else
		{
			Message(_T("\r\nError, FCD frequency NOT set."));
		}
	}
	else
	{
		Message(_T("\r\nError, FCD not found: is FCD correctly installed?"));
	}
	HIDClose(hWrite,hRead);
}

static void ResetFCD(void)
{
	TCHAR sz[200];
	TCHAR szMsg[400];
	BOOL b;
	HANDLE hWrite=INVALID_HANDLE_VALUE;
	HANDLE hRead=INVALID_HANDLE_VALUE;

	MessageClear();
	b=HIDOpen(_szVIDPID,sz,&hWrite,&hRead,sizeof(sz)/sizeof(TCHAR));
	_stprintf_s(szMsg,sizeof(szMsg)/sizeof(TCHAR),_T("HIDOpen of %s returns %d"),_szVIDPID,b);
	Message(szMsg);
	if (b)
	{
		unsigned __int8 u8;

		Message(_T("\r\nSuccess, FCD found."));
		u8=HIDResetFCD(hWrite,hRead);
		_stprintf_s(szMsg,sizeof(szMsg)/sizeof(TCHAR),_T("\r\nHIDReset returns %d"),u8);
		Message(szMsg);
		if (u8==1)
		{
			Message(_T("\r\nSuccess, FCD reset to bootloader."));
		}
		else
		{
			Message(_T("\r\nError, FCD NOT reset."));
		}
	}
	else
	{
		Message(_T("\r\nError, FCD not found: is FCD correctly installed?"));
	}
	HIDClose(hWrite,hRead);
}

static void WriteParameter(HWND hDlg,int nIdx) // nIdx is index into _acs of COMBOSTRUCT
{
	int n;
	unsigned __int8 u8;
	unsigned char auc[4];

	n=SendDlgItemMessage(hDlg,IDC_COMBOIFFILTER,CB_GETCURSEL,0,0);
	if (n!=-1)
	{
		u8=(unsigned __int8)n;

		WriteCommand(FCD_HID_CMD_SET_IF_FILTER,&u8,1);
	}

	u8=SendDlgItemMessage(hDlg,IDC_CHECKBIASTEE,BM_GETCHECK,0,0)==BST_CHECKED;
	WriteCommand(FCD_HID_CMD_SET_BIAS_TEE,&u8,1);

	u8=SendDlgItemMessage(hDlg,IDC_CHECKLNAENABLE,BM_GETCHECK,0,0)==BST_CHECKED;
	WriteCommand(FCD_HID_CMD_SET_LNA_GAIN,&u8,1);

	u8=SendDlgItemMessage(hDlg,IDC_CHECKMIXERGAIN,BM_GETCHECK,0,0)==BST_CHECKED;
	WriteCommand(FCD_HID_CMD_SET_MIXER_GAIN,&u8,1);

	n=GetScrollPos(GetDlgItem(hDlg,IDC_SCROLLBARFINEIFGAIN),SB_CTL);
	memcpy(auc+0,&n,2);
 	WriteCommand(FCD_HID_CMD_SET_IF_GAIN,auc,2);
}

static void WriteDevice(HWND hDlg)
{
	COMBOSTRUCT *pcs=_acs;
	int nIdx=0;

	while (pcs->pacis!=NULL)
	{
		WriteParameter(hDlg,nIdx);
		pcs++;
		nIdx++;
	}
}

static void SetDefaults(HWND hDlg)
{
	COMBOSTRUCT *pcs=_acs;

	SendDlgItemMessage(hDlg,IDC_COMBOIFFILTER,CB_SETCURSEL,(WPARAM)0,0);
	SendDlgItemMessage(hDlg,IDC_CHECKBIASTEE,BM_SETCHECK,BST_UNCHECKED,0);
	SendDlgItemMessage(hDlg,IDC_CHECKLNAENABLE,BM_SETCHECK,BST_CHECKED,0);
	SendDlgItemMessage(hDlg,IDC_CHECKMIXERGAIN,BM_SETCHECK,BST_CHECKED,0);
	SendDlgItemMessage(hDlg,IDC_SCROLLBARFINEIFGAIN,SBM_SETPOS,0,0);
}

static void RunSpectravue(HWND hDlg)
{
	{
		HWND hWnd=FindWindow(NULL,_T("SpectraVue Ver. 3.09   SpectraVue.ini"));

		if (hWnd!=NULL)
		{
			PostMessage(hWnd,WM_KEYDOWN,VK_F12,0);
			PostMessage(hWnd,WM_KEYUP,VK_F12,0x40000000);
/*
			PostMessage(hWnd,0xC2AE,0x80000098,0);

			PostMessage(hWnd,0xC2AE,0x80000048,1);
			PostMessage(hWnd,0xC2AE,0x80000088,0);
			PostMessage(hWnd,0xC2AE,0x80000080,8);
			PostMessage(hWnd,0xC2AE,0x800000B0,0);
			PostMessage(hWnd,0xC2AE,0x80000060,0x17700);
			PostMessage(hWnd,0xC2AE,0x80000050,0);
			PostMessage(hWnd,0xC2AE,0x80000010,0);
*/
		}
	}
	SetDefaults(hDlg);
	WriteDevice(hDlg);
	ReadDevice(hDlg); // No break!
}

static void SetDlgFreq(HWND hDlg,BOOL bMsgClear,BOOL bMsgError)
{
	int n;
	TCHAR sz[30];

	GetDlgItemText(hDlg,IDC_EDITFREQ,sz,sizeof(sz)-1);
	n=_ttoi(sz);

	if (n<100 || n>2500000)
	{
		if (bMsgError)
		{
			MessageBox(hDlg,_T("Target frequency must be between 100 and 2500000"),_T("Error"),MB_OK | MB_ICONEXCLAMATION);
		}
	}
	else
	{
		SetFreq(n,bMsgClear);
	}
}

static LRESULT CALLBACK MainDlg(HWND hDlg,UINT message,WPARAM wParam,LPARAM lParam)
{
	static HFONT hFont=INVALID_HANDLE_VALUE;
	static HBRUSH hBrush;
	static const COLORREF crEditFreqBk=RGB(0,0,0);
	static const COLORREF crEditFreq=RGB(0,255,0);
	static HBITMAP hBmp=INVALID_HANDLE_VALUE;

	switch (message) 
	{
		case WM_INITDIALOG:
			_hDlg=hDlg;
			hBrush = CreateSolidBrush(crEditFreqBk);
		  hFont=CreateFont (48, 0, 0, 0, FW_DONTCARE, FALSE, FALSE, FALSE, ANSI_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH | FF_SWISS, _T("Arial"));
			SendDlgItemMessage(hDlg,IDC_EDITFREQ,WM_SETFONT,(WPARAM)hFont,TRUE);
			SendDlgItemMessage(hDlg,IDC_STATICKHZ,WM_SETFONT,(WPARAM)hFont,TRUE);
			hBmp=LoadBitmap(_hInst,MAKEINTRESOURCE(IDB_BITMAP4)); 
			SendDlgItemMessage(hDlg,IDC_STATICPIC,STM_SETIMAGE,(WPARAM)IMAGE_BITMAP,(LPARAM)hBmp);

			{
				SCROLLINFO si;

				si.fMask=SIF_RANGE | SIF_PAGE | SIF_POS;
				si.cbSize=sizeof(si);
				si.nMax=59;
				si.nMin=0;
				si.nPage=1;
				si.nPos=0;

				{
					TCHAR sz[30]=_T("0");

					si.nPos=_ttoi(sz);
					SetDlgItemText(hDlg,IDC_EDITIFGAIN,sz);
				}
				SetScrollInfo(GetDlgItem(hDlg,IDC_SCROLLBARFINEIFGAIN),SB_CTL,&si,TRUE);
			}

			PopulateCombos(hDlg);			
			ReadDevice(hDlg);
			break;
		case WM_DESTROY:
			DeleteObject(hBmp);
			hBmp=INVALID_HANDLE_VALUE;
			DeleteObject(hFont);
			hFont=INVALID_HANDLE_VALUE;
			DeleteObject(hBrush);
			hBrush=INVALID_HANDLE_VALUE;
			_hDlg=NULL;
      return TRUE;
		case WM_DEVICECHANGE:
			MessageClear();
			ReadDevice(hDlg);
			break;
		case WM_CTLCOLOREDIT:
			if ((HWND)lParam==GetDlgItem(hDlg,IDC_EDITFREQ))
			{
				SetTextColor((HDC)wParam,crEditFreq);
				SetBkColor((HDC)wParam,crEditFreqBk);
				return (LRESULT)hBrush;
			}
			break;
		case WM_VSCROLL:
			{
				SCROLLINFO si;
				int nLineInc=1;

				si.cbSize=sizeof(si);
				si.fMask=SIF_ALL;
				GetScrollInfo((HWND)lParam,SB_CTL,&si);

				if ((GetAsyncKeyState(VK_CONTROL) & 0x8000)!=0)
				{
					nLineInc=si.nPage;
				}

        switch (LOWORD(wParam))
        {
					case SB_TOP:
						si.nPos=si.nMin;
            break;
		      case SB_LINEUP:
						si.nPos+=nLineInc;
	          break;
		      case SB_PAGEUP:
						si.nPos+=si.nPage;
	          break;
					case SB_LINEDOWN:
						si.nPos-=nLineInc;
						break;
					case SB_PAGEDOWN:
						si.nPos-=si.nPage;
						break;
	        case SB_THUMBPOSITION:
            si.nPos=HIWORD(wParam);
            break;
					case SB_THUMBTRACK:
						si.nPos=HIWORD(wParam);
						break;
					case SB_BOTTOM:
						si.nPos=si.nMax;
						break;
					case SB_ENDSCROLL:
						break;
				}
				if (si.nPos>si.nMax)
				{
					si.nPos=si.nMax;
				}
				if (si.nPos<si.nMin)
				{
					si.nPos=si.nMin;
				}

				SetScrollPos((HWND)lParam,SB_CTL,si.nPos,FALSE);

				{
					TCHAR sz[30];

					_itot_s(si.nPos,sz,sizeof(sz)/sizeof(TCHAR),10);

					switch (GetDlgCtrlID((HWND)lParam))
					{
						case IDC_SCROLLBARFINEIFGAIN:
							{
								TCHAR sz[30];

								_itot_s(si.nPos,sz,sizeof(sz)/sizeof(TCHAR),10);
							}
							SetDlgItemText(hDlg,IDC_EDITIFGAIN,sz);
							break;
					}
				}

				{
					unsigned char auc[4];
					int n;

					n=GetScrollPos(GetDlgItem(hDlg,IDC_SCROLLBARFINEIFGAIN),SB_CTL);
					memcpy(auc+0,&n,2);
 					WriteCommand(FCD_HID_CMD_SET_IF_GAIN,auc,2);
				}
			}
			break;
		case WM_COMMAND:
			switch (LOWORD(wParam))
			{
				case IDOK:
				case IDCANCEL:
				case ID_EXIT:
					EndDialog(hDlg, TRUE);
					break;

				case ID_SETFREQ:
					SetDlgFreq(hDlg,TRUE,TRUE);
					break;
				case ID_READDEVICE:
					ReadDevice(hDlg);
					break;
				case ID_WRITEDEVICE:
					WriteDevice(hDlg);
					ReadDevice(hDlg);
					break;
				case ID_DEFAULTS:
					SetDefaults(hDlg);
					WriteDevice(hDlg);
					ReadDevice(hDlg);
					break;
				case IDC_CHECKBIASTEE:
					if (HIWORD(wParam)==BN_CLICKED)
					{
						unsigned __int8 u8=SendDlgItemMessage(hDlg,IDC_CHECKBIASTEE,BM_GETCHECK,0,0)==BST_CHECKED;
						WriteCommand(FCD_HID_CMD_SET_BIAS_TEE,&u8,1);
					}
					break;
				case IDC_CHECKLNAENABLE:
					if (HIWORD(wParam)==BN_CLICKED)
					{
						unsigned __int8 u8=SendDlgItemMessage(hDlg,IDC_CHECKLNAENABLE,BM_GETCHECK,0,0)==BST_CHECKED;
						WriteCommand(FCD_HID_CMD_SET_LNA_GAIN,&u8,1);
					}
					break;
				case IDC_CHECKMIXERGAIN:
					if (HIWORD(wParam)==BN_CLICKED)
					{
						unsigned __int8 u8=SendDlgItemMessage(hDlg,IDC_CHECKMIXERGAIN,BM_GETCHECK,0,0)==BST_CHECKED;
						WriteCommand(FCD_HID_CMD_SET_MIXER_GAIN,&u8,1);
					}
					break;
				case IDC_COMBOIFFILTER:
					switch (HIWORD(wParam))
					{
						case CBN_SELCHANGE:
							{
								COMBOSTRUCT *pcs=_acs;
								int nIdx=-1; // Index into _acs array of combo box item

								while (pcs->pacis!=NULL && nIdx==-1)
								{
									if (pcs->nID==LOWORD(wParam))
									{
										nIdx=pcs-_acs;
									}
									else
									{
										pcs++;
									}
								}
								if (nIdx!=-1)
								{
									WriteParameter(hDlg,nIdx);
									ReadDevice(hDlg);
								}
							}
							break;
						default:
							break;
					}
					break;

				case ID_BOOTLOADER:
					ResetFCD();
					break;
				default:
					break;
			}
      return TRUE;

		default:
			break;
	}
  return FALSE;
}

int APIENTRY WinMain(HINSTANCE hInst,HINSTANCE hPrevInst,LPSTR lpCmdLine,int nCmdShow)
{
	_hInst=hInst;
	DialogBox(hInst,MAKEINTRESOURCE(IDD_MAIN),NULL,MainDlg);
  return 0;
}
